@@ -1,12 +1,12 @@
-#!/usr/bin/python2.7
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 #
 # Univention Management Console module:
 #  System Diagnosis UMC module
 #
-# Copyright 2014-2017 Univention GmbH
+# Copyright 2014-2021 Univention GmbH
 #
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -29,21 +29,28 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, seGe
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
 
 from os import listdir
 import os.path
 import traceback
 
+# strptime import is not threadsafe
+# Workaround: import early
+# Upstream bug: https://bugs.python.org/issue7980
+# UCS bug: https://forge.univention.org/bugzilla/show_bug.cgi?id=45585
+import _strptime
+
 from univention.management.console.modules import Base
 from univention.management.console.modules.decorators import simple_response, sanitize
 from univention.management.console.modules.sanitizers import PatternSanitizer, DictSanitizer, StringSanitizer
 from univention.management.console.modules.mixins import ProgressMixin
 from univention.management.console.log import MODULE
-
 from univention.management.console.modules.diagnostic import plugins
-
+from collections import OrderedDict
 from univention.lib.i18n import Translation
+
+_strptime
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 
@@ -90,14 +97,20 @@ class Instance(Base, ProgressMixin):
 		plugin=StringSanitizer(required=True),
 		args=DictSanitizer({})
 	)
-	@simple_response
+	@simple_response(with_progress=True)
 	def run(self, plugin, args=None):
 		plugin = self.get(plugin)
+		MODULE.process('Running %s' % (plugin,))
+		for line in plugin.run_descr:
+			MODULE.process(line)
 		args = args or {}
 
-		def thread(self, request):
-			return plugin.execute(self, **args)
-		return thread
+		return plugin.execute(self, **args)
+
+	def new_progress(self, *args, **kwargs):
+		progress = super(Instance, self).new_progress(*args, **kwargs)
+		progress.retry_after = 600
+		return progress
 
 	@sanitize(pattern=PatternSanitizer(default='.*'))
 	@simple_response
@@ -117,6 +130,7 @@ class Instance(Base, ProgressMixin):
 			except ImportError as exc:
 				MODULE.error('Could not load plugin %r: %r' % (plugin, exc))
 				raise
+		self.modules = OrderedDict(sorted(self.modules.items(), key=lambda t: t[0]))
 
 	def get(self, plugin):
 		return self.modules[plugin]
@@ -204,7 +218,6 @@ class Plugin(object):
 		+-- Critical
 		+-- ProblemFixed
 	"""
-
 	@property
 	def title(self):
 		u"""A title for the problem"""
@@ -220,6 +233,10 @@ class Plugin(object):
 		u"""Buttons which are displayed e.g. to automatically solve the problem"""
 		return list(getattr(self.module, 'buttons', []))
 
+	@property
+	def run_descr(self):
+		return list(getattr(self.module, 'run_descr', []))
+
 	@property
 	def popups(self):
 		u"""Buttons with pop ups"""
@@ -265,7 +282,7 @@ class Plugin(object):
 					errors.update(result)
 			except Problem:
 				raise
-			except:
+			except Exception:
 				raise Problem(traceback.format_exc())
 		except Problem as exc:
 			success = False
@@ -301,4 +318,4 @@ class Plugin(object):
 
 
 def main():
-	print 'TODO: someday implement?'
+	print('TODO: someday implement?')
