@@ -1,12 +1,12 @@
-#!/usr/bin/python2.7
+#!/usr/bin/python3
 # coding: utf-8
 #
 # Univention Management Console module:
 #  System Diagnosis UMC module
 #
-# Copyright 2016-2017 Univention GmbH
+# Copyright 2016-2021 Univention GmbH
 #
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -29,13 +29,13 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
 
 import struct
 import socket
 import random
 
-import ipaddr
+import ipaddress
 import dns.resolver
 import dns.exception
 from pyasn1.type import tag
@@ -46,18 +46,17 @@ from pyasn1.type import namedtype
 import pyasn1.codec.der.encoder
 import pyasn1.codec.der.decoder
 import pyasn1.error
-
 from univention.config_registry import handler_set as ucr_set
 import univention.config_registry
-from univention.management.console.modules.diagnostic import Warning, Critical, ProblemFixed
+from univention.management.console.modules.diagnostic import Warning, Critical, ProblemFixed, MODULE
 from univention.management.console.modules.diagnostic import util
 
 from univention.lib.i18n import Translation
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('KDC service check')
-description = _('The check for the KDC reachability was succesful.')
-
+description = ['The check for the KDC reachability was successful.']
+run_descr = ["Performs a KDC reachability check"]
 
 # This checks for the reachability of KDCs by sending a AS-REQ per TCP and UDP.
 # The AS-REQ is send with the fake user `kdc-reachability-check`. The KDCs will
@@ -90,63 +89,74 @@ def add_lo_to_samba_interfaces(umc_instance):
 
 	interfaces = configRegistry.get('samba/interfaces', '').split()
 	interfaces.append('lo')
+	MODULE.process('Setting samba/interfaces')
 	ucr_set(['samba/interfaces={}'.format(' '.join(interfaces))])
 	return run(umc_instance, retest=True)
 
 
 def reset_kerberos_kdc(umc_instance):
+	MODULE.process('Resetting kerberos/kdc=127.0.0.1')
 	ucr_set(['kerberos/kdc=127.0.0.1'])
 	return run(umc_instance, retest=True)
 
 
+description = _('ph ')
 actions = {
 	'add_lo_to_samba_interfaces': add_lo_to_samba_interfaces,
 	'reset_kerberos_kdc': reset_kerberos_kdc,
 }
 
 
-def _c(n, t):
-	return t.clone(tagSet=t.tagSet + tag.Tag(tag.tagClassContext, tag.tagFormatSimple, n))
+def _sequence_component(name, tag_value, type):
+	return namedtype.NamedType(name, type.subtype(
+		explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, tag_value),))
+
+
+def _sequence_optional_component(name, tag_value, type):
+	return namedtype.OptionalNamedType(name, type.subtype(
+		explicitTag=tag.Tag(tag.tagClassContext, tag.tagFormatSimple, tag_value),))
 
 
 class PrincipalName(univ.Sequence):
 	componentType = namedtype.NamedTypes(
-		namedtype.NamedType('name-type', _c(0, univ.Integer())),
-		namedtype.NamedType('name-string', _c(1, univ.SequenceOf(componentType=char.GeneralString()))))
+		_sequence_component('name-type', 0, univ.Integer()),
+		_sequence_component('name-string', 1, univ.SequenceOf(componentType=char.GeneralString()))
+	)
 
 
 class KdcReqBody(univ.Sequence):
-	tagSet = univ.Sequence.tagSet + tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
 	componentType = namedtype.NamedTypes(
-		namedtype.NamedType('kdc-options', _c(0, univ.BitString())),
-		namedtype.OptionalNamedType('cname', _c(1, PrincipalName())),
-		namedtype.NamedType('realm', _c(2, char.GeneralString())),
-		namedtype.OptionalNamedType('sname', _c(3, PrincipalName())),
-		namedtype.NamedType('till', _c(5, useful.GeneralizedTime())),
-		namedtype.NamedType('nonce', _c(7, univ.Integer())),
-		namedtype.NamedType('etype', _c(8, univ.SequenceOf(componentType=univ.Integer()))))
+		_sequence_component('kdc-options', 0, univ.BitString()),
+		_sequence_optional_component('cname', 1, PrincipalName()),
+		_sequence_component('realm', 2, char.GeneralString()),
+		_sequence_optional_component('sname', 3, PrincipalName()),
+		_sequence_optional_component('from', 4, useful.GeneralizedTime()),
+		_sequence_component('till', 5, useful.GeneralizedTime()),
+		_sequence_component('nonce', 7, univ.Integer()),
+		_sequence_component('etype', 8, univ.SequenceOf(componentType=univ.Integer())))
 
 
 class PAData(univ.Sequence):
+	tagSet = univ.Sequence.tagSet + tag.Tag(tag.tagClassContext, tag.tagFormatSimple, 4)
 	componentType = namedtype.NamedTypes(
-		namedtype.NamedType('padata-type', _c(1, univ.Integer())),
-		namedtype.NamedType('padata-value', _c(2, univ.OctetString())))
+		_sequence_component('padata-type', 1, univ.Integer()),
+		_sequence_component('padata-value', 2, univ.OctetString()))
 
 
 class AsReq(univ.Sequence):
 	tagSet = univ.Sequence.tagSet + tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 10)
 	componentType = namedtype.NamedTypes(
-		namedtype.NamedType('pvno', _c(1, univ.Integer())),
-		namedtype.NamedType('msg-type', _c(2, univ.Integer())),
-		namedtype.NamedType('padata', _c(3, univ.SequenceOf(componentType=PAData()))),
-		namedtype.NamedType('req-body', KdcReqBody()))
+		_sequence_component('pvno', 1, univ.Integer()),
+		_sequence_component('msg-type', 2, univ.Integer()),
+		_sequence_component('padata', 3, univ.SequenceOf(componentType=PAData())),
+		_sequence_component('req-body', 4, KdcReqBody()))
 
 
 class AsRep(univ.Sequence):
 	tagSet = univ.Sequence.tagSet + tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 11)
 	componentType = namedtype.NamedTypes(
-		namedtype.NamedType('pvno', _c(0, univ.Integer())),
-		namedtype.NamedType('msg-type', _c(1, univ.Integer()))
+		_sequence_component('pvno', 0, univ.Integer()),
+		_sequence_component('msg-type', 1, univ.Integer())
 		# some more omitted
 	)
 
@@ -154,8 +164,8 @@ class AsRep(univ.Sequence):
 class KrbError(univ.Sequence):
 	tagSet = univ.Sequence.tagSet + tag.Tag(tag.tagClassApplication, tag.tagFormatSimple, 30)
 	componentType = namedtype.NamedTypes(
-		namedtype.NamedType('pvno', _c(0, univ.Integer())),
-		namedtype.NamedType('msg-type', _c(1, univ.Integer()))
+		_sequence_component('pvno', 0, univ.Integer()),
+		_sequence_component('msg-type', 1, univ.Integer())
 		# some more omitted
 	)
 
@@ -176,33 +186,48 @@ class EmptyResponse(KerberosException):
 	pass
 
 
+class Principal(object):
+
+	def __init__(self, name, realm, princ_type):
+		self.type = princ_type
+		self.realm = realm
+		self.components = [name, realm]
+
+	def components_to_asn1(self, name):
+		name.setComponentByName('name-type', int(self.type))
+		strings = name.setComponentByName('name-string').getComponentByName('name-string')
+		for i, c in enumerate(self.components):
+			strings.setComponentByPosition(i, c)
+		return name
+
+
+def seq_set(seq, name, builder=None):
+	component = seq.setComponentByName(name).getComponentByName(name)
+	if builder is not None:
+		seq.setComponentByName(name, builder(component))
+	else:
+		seq.setComponentByName(name)
+	return seq.getComponentByName(name)
+
+
 def build_kerberos_request(target_realm, user_name):
-	req_body = KdcReqBody()
-	req_body['kdc-options'] = "'01010000100000000000000000000000'B"
-
-	req_body['cname'] = None
-	req_body['cname']['name-type'] = 1  # NT_PRINCIPAL
-	req_body['cname']['name-string'] = None
-	req_body['cname']['name-string'][0] = user_name
-
-	req_body['realm'] = target_realm
-
-	req_body['sname'] = None
-	req_body['sname']['name-type'] = 2  # NT_SRV_INST
-	req_body['sname']['name-string'] = None
-	req_body['sname']['name-string'][0] = 'krbtgt'
-	req_body['sname']['name-string'][1] = target_realm
-
-	req_body['till'] = '19700101000000Z'
-	req_body['nonce'] = random.SystemRandom().getrandbits(31)
-	req_body['etype'] = None
-	req_body['etype'][0] = 18  # AES256_CTS_HMAC_SHA1_96
 
 	as_req = AsReq()
 	as_req['pvno'] = 5
 	as_req['msg-type'] = 10  # AS-REQ
-	as_req['padata'] = None
-	as_req['req-body'] = req_body
+	as_req['padata'] = univ.noValue
+
+	serverName = Principal('krbtgt', target_realm, 1)
+	clientName = Principal(user_name, target_realm, 1)
+
+	req_body = seq_set(as_req, 'req-body')
+	req_body['kdc-options'] = [0 for i in range(0, 32)]
+	seq_set(req_body, 'sname', serverName.components_to_asn1)
+	seq_set(req_body, 'cname', clientName.components_to_asn1)
+
+	req_body['realm'] = target_realm
+	req_body['till'] = '19700101000000Z'
+	req_body['nonce'] = random.SystemRandom().getrandbits(31)
 
 	return pyasn1.codec.der.encoder.encode(as_req)
 
@@ -220,20 +245,20 @@ def send_and_receive(kdc, port, protocol, as_req):
 	try:
 		sock.connect((kdc, port))
 		sock.sendall(packed)
-	except socket.error, socket.timeout:
+	except (socket.error, socket.timeout):
 		sock.close()
 		raise ServerUnreachable()
 
-	received = ''
+	received = b''
 	num_received = 0
 	if protocol == 'udp':  # fake the length field
-		received += '\x00\x00\x00\x00'
+		received += b'\x00\x00\x00\x00'
 		num_received += 4
 	while num_received < 128:
 		try:
 			(buf, addr) = sock.recvfrom(128)
-		except socket.error, socket.timeout:
-			buf = ''
+		except (socket.error, socket.timeout):
+			buf = b''
 		if not buf:
 			break
 		received += buf
@@ -247,12 +272,24 @@ def send_and_receive(kdc, port, protocol, as_req):
 
 def probe_kdc(kdc, port, protocol, target_realm, user_name):
 	request = build_kerberos_request(target_realm, user_name)
-
+	if protocol == 'udp':
+		MODULE.process("Trying to contact KDC %s on udp port %d Similar to running: nmap %s -sU -p %d" % (kdc, port, kdc, port))
+	else:
+		MODULE.process("Trying to contact KDC %s on tcp port %d Similar to running: nmap %s -sT -p %d" % (kdc, port, kdc, port))
 	try:
 		received = send_and_receive(kdc, port, protocol, request)
 	except KerberosException:
 		return False
 
+	if target_realm.encode('UTF-8') in received:
+		return True
+
+	return False
+
+	# this no longer works with >= 4.3, ??
+	# I think the new pyasn1 version might need the full asn1Spec to work?:
+	# http://snmplabs.com/pyasn1/changelog.html
+	# Keyword: require strict two-zeros sentinel encoding
 	try:
 		(error, _sub) = pyasn1.codec.der.decoder.decode(received, asn1Spec=KrbError())
 	except pyasn1.error.PyAsn1Error:
@@ -296,19 +333,19 @@ def run(_umc_instance, retest=False):
 		kdc_to_check = [(kdc, 88, 'tcp') for kdc in kdc_fqds]
 		kdc_to_check.extend((kdc, 88, 'udp') for kdc in kdc_fqds)
 
-	kdc_reachabe = [(probe_kdc(kdc, port, protocol, target_realm, user_name),
-		(kdc, port, protocol)) for (kdc, port, protocol) in kdc_to_check]
-	reachable_kdc = [(kdc, port, protocol) for (reachable, (kdc, port, protocol))
+	kdc_reachabe = [(probe_kdc(kdc, port, protocol, target_realm, user_name), (kdc, port, protocol)) for (kdc, port, protocol) in kdc_to_check]
+	reachable_kdc = [
+		(kdc, port, protocol) for (reachable, (kdc, port, protocol))
 		in kdc_reachabe if reachable]
-	unreachable_kdc = [(kdc, port, protocol) for (reachable, (kdc, port, protocol))
+	unreachable_kdc = [
+		(kdc, port, protocol) for (reachable, (kdc, port, protocol))
 		in kdc_reachabe if not reachable]
 
 	error_descriptions = list()
 
 	if unreachable_kdc:
 		error = _('The following KDCs were unreachable: {}')
-		unreach_string = ('{} {}:{}'.format(protocol, kdc, port)
-			for (kdc, port, protocol) in unreachable_kdc)
+		unreach_string = ('{} {}:{}'.format(protocol, kdc, port) for (kdc, port, protocol) in unreachable_kdc)
 		error_descriptions.append(error.format(', '.join(unreach_string)))
 
 	if not reachable_kdc:
@@ -318,13 +355,14 @@ def run(_umc_instance, retest=False):
 			local_included = False
 			for interface in configRegistry.get('samba/interfaces', '').split():
 				try:
-					addr = ipaddr.IPAddress(interface)
+					addr = ipaddress.ip_address(u'%s' % (interface,))
 				except ValueError:
 					local_included |= interface == 'lo'
 				else:
 					local_included |= addr.is_loopback or addr.is_unspecified
-			error = _('samba/interfaces does not contain lo, 127.0.0.1 or 0.0.0.0.')
-			error_descriptions.append(error)
+			if not local_included:
+				error = _('samba/interfaces does not contain lo, 127.0.0.1 or 0.0.0.0.')
+				error_descriptions.append(error)
 
 			description = '\n'.join(error_descriptions)
 			buttons = [{
@@ -342,6 +380,7 @@ def run(_umc_instance, retest=False):
 
 	if error_descriptions:
 		error = '\n'.join(error_descriptions)
+		MODULE.error(error)
 		raise Warning(description=error)
 
 	if retest:
