@@ -1,12 +1,12 @@
-#!/usr/bin/python2.7
+#!/usr/bin/python3
 # coding: utf-8
 #
 # Univention Management Console module:
 #  System Diagnosis UMC module
 #
-# Copyright 2016-2017 Univention GmbH
+# Copyright 2016-2021 Univention GmbH
 #
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -29,16 +29,16 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
 
-from univention.management.console.modules.diagnostic import Critical, ProblemFixed
+from univention.management.console.modules.diagnostic import Critical, ProblemFixed, MODULE
 from univention.management.console.modules.diagnostic import util
-
 from univention.lib.i18n import Translation
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check local AD database for errors')
 description = _('No errors found.'),
+run_descr = ['This can be checked by running: samba-tool dbcheck']
 
 
 def run_samba_tool_dbcheck_fix(umc_instance):
@@ -47,10 +47,11 @@ def run_samba_tool_dbcheck_fix(umc_instance):
 
 	cmd = ['samba-tool', 'dbcheck', '--fix', '--cross-ncs', '--yes']
 	(success, output) = util.run_with_output(cmd)
-	if success:
-		fix_log = [_('`{cmd}` failed.').format(cmd=' '.join(cmd))]
-	else:
-		fix_log = [_('`{cmd}` succeeded.').format(cmd=' '.join(cmd))]
+
+	cmd_string = ' '.join(cmd)
+	MODULE.process('Output of %s:\n%s' % (cmd_string, output))
+	fix_log = [_('Output of `{cmd}`:').format(cmd=cmd_string)]
+
 	fix_log.append(output.decode('utf-8', 'replace'))
 	run(umc_instance, rerun=True, fix_log='\n'.join(fix_log))
 
@@ -75,8 +76,8 @@ def run(_umc_instance, rerun=False, fix_log=''):
 
 	cmd = ['samba-tool', 'dbcheck']
 	(success, output) = util.run_with_output(cmd)
-	if not success:
-		error = _('`samba-tool dbcheck` returned a problem with the local AD database.')
+	if [x for x in output.split('\n') if x.startswith("ERROR:")]:
+		error = _('`samba-tool dbcheck` found an error in the local AD database.')
 		error_descriptions.append(error)
 		error_descriptions.append(output)
 		if not rerun:
@@ -85,9 +86,9 @@ def run(_umc_instance, rerun=False, fix_log=''):
 		raise Critical(description='\n'.join(error_descriptions), buttons=buttons)
 
 	if rerun:
-		fixed = _('`samba-tool dbcheck` found no problems with the local AD database.')
+		fixed = _('`samba-tool dbcheck` found no errors in the local AD database.')
 		error_descriptions.append(fixed)
-		error_descriptions.append(output)
+		MODULE.error('\n'.join(error_descriptions))
 		raise ProblemFixed(description='\n'.join(error_descriptions))
 
 
